# jinit.tcl - common application initialisation procedures
#
###############################################################################
#  Copyright 1992-2001 by Jay Sekora.  This file may be freely distributed,   #
#  modified or unmodified, for any purpose, provided that this copyright      #
#  notice is retained verbatim in all copies and no attempt is made to        #
#  obscure the authorship of this file.  If you distribute any modified       #
#  versions, I ask, but do not require, that you clearly mark any changes     #
#  you make as such and that you provide your users with instructions for     #
#  getting the original sources.                                              #
###############################################################################

package provide jstools 0.1

# following has side-effect of making sure ::jstools namespace is defined
namespace eval ::jstools {
  namespace export {[a-z]*}
  catch {
    package require jldb
    # following likely to fail, because done by other libraries too
    namespace import ::jldb::shortcuts::*
  }
}

######################################################################
# global variables:
#
global J_PREFS env
if {! [info exists J_PREFS(autoposition)]} {set J_PREFS(autoposition) 0}
if {! [info exists J_PREFS(confirm)]} {set J_PREFS(confirm) 1}
#
######################################################################


######################################################################
# ::jstools::jstools_init - common initialisation for jstools
######################################################################

proc ::jstools::jstools_init { {app jstools} } {
  if {[catch {package require Tk 8.0}]} {
    tkerror {This application requires at least Tk 8.0.}
    exit 1
  }
  
  package require jldb
  
  global J_PREFS			;# user preferences
  
  global JSTOOLS_VERSION
  set JSTOOLS_VERSION {2001.02.10}
  
  global tk_library
  global env
  global NAME HOME
  
  set NAME {}
  set HOME /
  
  if {![info exists env(USER)]} {
    if [info exists env(LOGNAME)] {
      set env(USER) $env(LOGNAME)
    } else {
      set env(USER) [file tail $env(HOME)]
    }
  }
  catch {set NAME $env(USER)}
  catch {set HOME $env(HOME)}
  
  ::jstools::wm_client			;# set client hostname
  ::jstools::wm_command			;# set client command string
  
  j:pref:create_global_prefs		;# set up standard global prefs
  j:read_global_prefs			;# read standard jstools preferences
  ::jldb::init jstools			;# read in language database for library
  j:pref:create_global_prefs		;# again with language set (inefficient)
  j:source_config jstoolsrc.tcl		;# read cross-application preferences
  catch {				;# read cross-application X defaults:
    option readfile [glob -nocomplain ~/.tk/jstools.ad] userDefault
  }
  if {"x$app" != "xjstools"} {		;# application-specific initialisation
    ::jldb::init $app			;# read in language database for app
    catch {				;# read application-specific X defaults:      option readfile [glob -nocomplain ~/.tk/$app.ad] userDefault
    }
    ###					;# could read preferences here?
  }
  
  # following use users prefs, bindings files:
  j:tb:init Text			;# uses prefs, ~/.tk/textbindings.tcl
  j:eb:init Entry			;# uses prefs, ~/.tk/entrybindings.tcl
  
  ::jstools::command_register \
    ::jstools::cmd::prompt_tcl {Issue Tcl Command...}
  ::jstools::command_register \
    ::jstools::cmd::prompt_unix {Issue Unix Command...}
  ::jstools::command_register \
    ::jstools::cmd::global_pref_panel {Global Preferences...}
  
  ::jstools::command_register j:cmd:text_cut {Cut}
  ::jstools::command_register j:cmd:text_copy {Copy}
  ::jstools::command_register j:cmd:text_paste {Paste}
}



